/*
 * FileDataStore.java
 */

package catalog.datastore;

import java.lang.*;
import java.io.*;
import java.sql.*;
import java.util.*;
import catalog.compactdisk.*;
import catalog.CompactDiskCollection;


/**
 * An implementation class for DataStore interface which provide file type of
 * data store.
 */
public class DataBaseDataStore
  implements DataStore{

  /**
   * Singleton instance of DataStore
   */
  private static DataStore m_instance;

  /**
   * Singleton instance of Connection
   */
  private static Connection con;


  /**
   * Method to open the data store.
   * <p><b>pre:</b> True.
   * <p><b>post:</b>The datastore is opened.
   */
  public void open()
    throws OpenDataStoreException
  {
    try
    {
      // Load database driver
      Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");

      // Establish connection
      con = DriverManager.getConnection("jdbc:odbc:catalog");
    }
    catch(ClassNotFoundException e)
    {
      throw new OpenDataStoreException(e.toString());
    }
    catch(SQLException e)
    {
      throw new OpenDataStoreException(e.toString());
    }
  }

  /**
   * Method to close the data store.
   * <p><b>pre:</b> The datastore is open.
   * <p><b>post:</b>The data store is closed.
   */
  public void close()
    throws CloseDataStoreException
  {
    try
    {
      con.close();
      m_instance = null;
    }
    catch(SQLException e)
    {
      throw new CloseDataStoreException(e.toString());
    }
  }

  /**
   * Accessor to the singleton instance.
   * <p><b>pre:</b> True.
   * <p><b>post:</b>A reference to the singleton object returned.
   */
  public static DataStore getInstance()
  {
    if (m_instance == null)
      m_instance = new DataBaseDataStore();
    return m_instance;
  }

  /**
   * Report the details of the connection.
   * <p><b>pre:</b> A connection has been established by a successful call
   *       to method open().
   * <p><b>post:</b>Details of the connection are displayed to standard output.
   */
  public void reportConnectionDetails()
  {
    try
    {
      if(!con.isClosed())
        System.out.println("Database connected");

      DatabaseMetaData dbmd = con.getMetaData();
      System.out.println("Catalog name  : " + dbmd.getCatalogs());
      System.out.println("Product name  : " + dbmd.getDatabaseProductName());
      System.out.println("Driver name   : " + dbmd.getDriverName());
      System.out.println("Version name  : " + dbmd.getDriverVersion());
      System.out.println("URL           : " + dbmd.getURL());
    }
    catch(SQLException e)
    {
      System.err.println(e);
    }
  }

  /**
   * Used to load all the CompactDisk records from the database.
   * <p><b>pre:</b>The database datastore is open.
   * <p><b>post:</b>All cd's are loaded from the database datastore into the
   *      Collection. The maximum product code of the Collection is set.
   */
  public CompactDiskCollection
    loadCompactDiskCollection(CompactDiskCollection cdCollection)
      throws LoadDataStoreException
  {
    try
    {
      // Create and send statement
      String sQuery = "SELECT product_code, artist, title, year, cover, stock " +
                      "FROM JoinAll " +
                      "ORDER BY product_code ASC";
      Statement stmt = con.createStatement();
      ResultSet rslt = stmt.executeQuery(sQuery);

      while(rslt.next())
      {
        cdCollection.addCompactDisk
        (
          new CompactDisk
          (
            rslt.getInt("product_code"),
            rslt.getString("artist"),
            rslt.getString("title"),
            rslt.getString("year"),
            rslt.getString("cover"),
            rslt.getInt("stock")
          )
        );
      }
    }
    catch(SQLException e)
    {
      throw new LoadDataStoreException(e.toString());
    }
    return cdCollection;
  }

  /**
   * Used to save a CompactDisk object to the data store.
   * <p><b>pre:</b> The datastore is open. The given cd parameter is valid.
   * <p><b>post:</b>The given cd object has been written to the datastore.
   */
  public void saveCompactDisk(CompactDisk cd)
    throws SaveDataStoreException
  {
    CompactDiskCollection cds = CompactDiskCollection.getInstance();
    if (!(cds.containsTitle(cd.getArtist(), cd.getTitle())))
    {
      try
      {
	  	  Statement stmt1, stmt2, stmt3;
	    	PreparedStatement ps1, ps2, ps3;
        String sql1, sql2, sql3;

    		sql1 = "INSERT INTO cd_cover(product_code,cover) VALUES (?,?)";
		  	ps1 = con.prepareStatement(sql1);
			  stmt1 = con.createStatement();

  			// inserting records
	  		ps1.setInt(1, cd.getProductCode());
		  	ps1.setString(2, cd.getCover());
  			ps1.executeUpdate();

    		sql2 = "INSERT INTO compact_disk(product_code,artist,title,year) VALUES (?,?,?,?)";
		  	ps2 = con.prepareStatement(sql2);
			  stmt2 = con.createStatement();

  			// inserting records
	  		ps2.setInt(1, cd.getProductCode());
		  	ps2.setString(2, cd.getArtist());
			  ps2.setString(3, cd.getTitle());
  			ps2.setString(4, cd.getYear());
	  		ps2.executeUpdate();

  	  	sql3 = "INSERT INTO stock_level(product_code,stock) VALUES (?,?)";
  			ps3 = con.prepareStatement(sql3);
	  		stmt3 = con.createStatement();

		  	// inserting records
  			ps3.setInt(1, cd.getProductCode());
	  		ps3.setInt(2, cd.getStockCount());
		  	ps3.executeUpdate();

        con.commit();
      }
      catch(SQLException e)
      {
        System.err.println(e.getErrorCode());
      }
    }
    else
    {
      throw new SaveDataStoreException("CD record aleardy exist in Database");
    }
  }

  /**
   * Used to update a CompactDisk object in the data store.
   * <p><b>pre:</b> The datastore is open. The compact disk parameter is valid.
   * <p><b>post:</b>The stock count in the datastore is updated for the
   *       CompactDisk details given.
   */
  public void updateCompactDisk(CompactDisk cd)
    throws SaveDataStoreException
  {
    try
    {
		  Statement stmt;
	  	PreparedStatement ps;
      String sql;

  		sql = "UPDATE stock_level SET stock=? WHERE product_code=?";
			ps = con.prepareStatement(sql);
			stmt = con.createStatement();

			// inserting records
			ps.setInt(1, cd.getStockCount());
      ps.setInt(2, cd.getProductCode());
			ps.executeUpdate();
    }
    catch(SQLException e)
    {
      throw new SaveDataStoreException(e.toString());
    }
  }
} // eof.
