/*
 * FileDataStore.java
 */

package catalog.datastore;

import java.io.*;
import catalog.compactdisk.*;
import java.io.RandomAccessFile;
import java.util.StringTokenizer;
import catalog.CompactDiskCollection;


/**
 * An implementation class for FileDataStore interface which provide file type of
 * data store.
 */
public class FileDataStore
  implements DataStore{

  private static DataStore m_instance;
  private static RandomAccessFile m_file;
  private static long file_ptr;

  /**
   * Method to open the file data store.
   * <p><b>pre:</b> True.
   * <p><b>post:</b>The file datastore is opened.
   */
  public void open()
    throws OpenDataStoreException
  {
    try
    {
      File my_file = new File("catalog\\datastore\\catalog.dat");
      if (!my_file.isFile())
        throw new FileNotFoundException();

      m_file = new RandomAccessFile(my_file, "rw");

      // get end of file pointer value
      file_ptr = m_file.length();
    }
    catch (FileNotFoundException e)
    {
      throw new OpenDataStoreException(e.toString());
    }
    catch(IOException e)
    {
      throw new OpenDataStoreException(e.toString());
    }
  }

  /**
   * Method to close the file data store.
   * <p><b>pre:</b> The file datastore is open.
   * <p><b>post:</b>The file data store is closed.
   */
  public void close()
    throws CloseDataStoreException
  {
    try
    {
      m_file.close();
    }
    catch(IOException e)
    {
      throw new CloseDataStoreException(e.toString());
    }
  }

  /**
   * Accessor to the singleton instance.
   * <p><b>pre:</b> True.
   * <p><b>post:</b>A reference to the singleton object returned.
   */
  public static DataStore getInstance()
  {
    if (m_instance == null)
      m_instance = new FileDataStore();
    return m_instance;
  }

  /**
   * Report the details of the connection.
   * <p><b>pre:</b> A connection has been established by a successful call
   *       to method open().
   * <p><b>post:</b>Details of the connection are displayed to standard output.
   */
  public void reportConnectionDetails()
  {
    try
    {
      System.out.println("File length is : " + m_file.length());
      System.out.println("File pointer location is : " + m_file.getFilePointer());
    }
    catch (IOException e)
    {
      System.err.println(e);
    }
  }

  /**
   * Used to load all the CompactDisk records from the file.
   * <p><b>pre:</b>The file datastore is open.
   * <p><b>post:</b>All cd's are loaded from the file datastore into the
   *      Collection. The maximum product code of the Collection is set.
   */
  public CompactDiskCollection
    loadCompactDiskCollection(CompactDiskCollection cdCollection)
      throws LoadDataStoreException
  {
    boolean EOF = false;
    String s_productCode, s_artistName, s_compactDiskTitle, s_yearOfPublication;
    String s_cover, s_stockCount;

    try
    {
      // set the file pointer value to 0 to read from begining of file
      file_ptr = 0;
      m_file.seek(file_ptr);
      while(!EOF)
      {
        String temp = m_file.readLine();

        if (temp!=null)
        {
          String[] data_read = new String[6];
          StringTokenizer str = new StringTokenizer(temp, ";");

          s_productCode = str.nextToken();
          s_artistName = str.nextToken();
          s_compactDiskTitle = str.nextToken();
          s_yearOfPublication = str.nextToken();
          s_cover = str.nextToken();
          s_stockCount = str.nextToken();

          // to eliminate space in the last token for string to int conversion
          StringTokenizer delete_space = new StringTokenizer(s_stockCount, " ");
          s_stockCount = delete_space.nextToken();

          // Add compact disk (from text file) into compact disk collection
          cdCollection.addCompactDisk
          (
            new CompactDisk
            (
              new Integer(s_productCode).intValue(),
              s_artistName,
              s_compactDiskTitle,
              s_yearOfPublication,
              s_cover,
              new Integer(s_stockCount).intValue()
            )
          );
        }
        else
          EOF = true;
      }
    }
    catch(IOException e)
    {
      throw new LoadDataStoreException(e.toString());
    }
    return cdCollection;
  }


  /**
   * Used to save a CompactDisk object to the file data store.
   * <p><b>pre:</b> The file datastore is open. The given cd parameter is valid.
   * <p><b>post:</b>The given cd object has been written to the file datastore.
   */
  public void saveCompactDisk(CompactDisk cd)
    throws SaveDataStoreException
  {
    CompactDiskCollection cds = CompactDiskCollection.getInstance();
    if (!(cds.containsTitle(cd.getArtist(), cd.getTitle())))
    {
      try
      {
        // set the value of file pointer to the end of file to write data
        file_ptr = m_file.length();
        m_file.seek(file_ptr);

        String spaces = new String();

        int size = new Integer(cd.getStockCount()).toString().length();

        if (size==1)
          spaces = spaces + "    ";
        else if (size==2)
          spaces = spaces + "   ";
        else if (size==3)
          spaces = spaces + "  ";
        else if (size==4)
          spaces = spaces + " ";


        String content = new Integer(cd.getProductCode()).toString() + ";" +
                         cd.getArtist() + ";" + cd.getTitle() + ";" +
                         cd.getYear() + ";" + cd.getCover() + ";" +
                         new Integer(cd.getStockCount()).toString() +
                         spaces + System.getProperty("line.separator");
        m_file.writeBytes(content);
      }
      catch(IOException e)
      {
        throw new SaveDataStoreException(e.toString());
      }
    }
    else
    {
      throw new SaveDataStoreException("CD record aleardy exist in text file");
    }
  }

  /**
   * Used to update a CompactDisk object in the file data store.
   * <p><b>pre:</b> The file datastore is open. The compact disk parameter
           is valid.
   * <p><b>post:</b>The stock count in the file datastore is updated for the
   *       CompactDisk details given.
   */
  public void updateCompactDisk(CompactDisk cd)
    throws SaveDataStoreException
  {
    String temp;
    try
    {
      // set the value of file pointer to the end of file to write data
      file_ptr = 0;
      m_file.seek(file_ptr);

      // This portion of code uses the product code to create a loop to search
      // for the proper file pointer value for stock count of a particular CD
      for (int line=1000; line < cd.getProductCode(); line++)
      {
        temp = m_file.readLine();
        file_ptr = file_ptr + temp.length() + 2;
      }

      // Find the position of the stock count value in the row of data
      // and calculate the absolute file pointer value for stock count
      temp = m_file.readLine();

      file_ptr = file_ptr + temp.length() - 5;

      String stock = new Integer(cd.getStockCount()).toString();

      // Overide the previous value on the current location
      int size = stock.length();

      if (size==1)
        stock = stock + "    ";
      else if (size==2)
        stock = stock + "   ";
      else if (size==3)
        stock = stock + "  ";
      else if (size==4)
        stock = stock + " ";

      m_file.seek(file_ptr);
      m_file.writeBytes(stock);
    }
    catch(IOException e)
    {
      throw new SaveDataStoreException(e.toString());
    }
  }

} // eof.